@@ -1,3 +1,9 @@
+/*!
+# Error types
+
+This module holds the error types returned by the various functions of this
+crate.
+*/
 use std::io;
 use std::sync::Arc;
 use std::fmt;
@@ -5,6 +11,8 @@ use std::error;
 use std::ops::Deref;
 use std::result::Result as StdResult;
 
+use rxml_validation::{Error as ValidationError};
+
 pub const ERRCTX_UNKNOWN: &'static str = "in unknown context";
 pub const ERRCTX_TEXT: &'static str = "in text node";
 pub const ERRCTX_ATTVAL: &'static str = "in attribute value";
@@ -73,6 +81,8 @@ pub enum WFError {
 	ElementMismatch,
 }
 
+impl error::Error for WFError {}
+
 impl ErrorWithContext for WFError {
 	fn with_context(self, ctx: &'static str) -> WFError {
 		match self {
@@ -133,6 +143,15 @@ impl fmt::Display for WFError {
 	}
 }
 
+impl From<ValidationError> for WFError {
+	fn from(other: ValidationError) -> Self {
+		match other {
+			ValidationError::EmptyName => Self::InvalidSyntax("Name must have at least one Char"),
+			ValidationError::InvalidChar(ch) => Self::UnexpectedChar(ERRCTX_UNKNOWN, ch, None),
+		}
+	}
+}
+
 /// Violation of a namespace-well-formedness constraint or the Namespaces for
 /// XML 1.0 grammar.
 #[derive(Debug, Clone, PartialEq)]
@@ -162,6 +181,8 @@ pub enum NWFError {
 	EmptyNamespaceUri,
 }
 
+impl error::Error for NWFError {}
+
 impl ErrorWithContext for NWFError {
 	fn with_context(self, ctx: &'static str) -> NWFError {
 		match self {
