@@ -2,17 +2,18 @@
 # XML 1.0 Lexer
 */
 // needed for trait bounds
-#[allow(unused_imports)]
-use std::io;
+use std::convert::TryInto;
 use std::fmt;
 
 mod read;
+mod ranges;
 
-use crate::selectors::*;
+use rxml_validation::selectors::*;
 use read::{Endpoint};
 use crate::error::{Error, Result, WFError};
 use crate::error::*;
 use crate::strings::*;
+use ranges::*;
 
 pub use read::{Utf8Char, CodepointRead, DecodingReader};
 
@@ -58,7 +59,7 @@ impl TokenMetrics {
 	}
 
 	// for use in parser unit tests
-	#[allow(dead_code)]
+    #[cfg(test)]
 	pub(crate) const fn new(start: usize, end: usize) -> TokenMetrics {
 		TokenMetrics{start: start, end: end}
 	}
@@ -376,10 +377,9 @@ impl LexerOptions {
 	/// Constructs default lexer options.
 	///
 	/// The defaults are implementation-defined and should not be relied upon.
+	#[deprecated(since = "0.4.0", note = "use the Default trait implementation instead")]
 	pub fn defaults() -> LexerOptions {
-		LexerOptions{
-			max_token_length: 8192,
-		}
+		Self::default()
 	}
 
 	/// Set the [`LexerOptions::max_token_length`] value.
@@ -388,7 +388,7 @@ impl LexerOptions {
 	///
 	/// ```
 	/// use rxml::{Lexer, LexerOptions};
-	/// let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(1024));
+	/// let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(1024));
 	/// ```
 	pub fn max_token_length(mut self, v: usize) -> LexerOptions {
 		self.max_token_length = v;
@@ -396,6 +396,17 @@ impl LexerOptions {
 	}
 }
 
+impl Default for LexerOptions {
+	/// Constructs default lexer options.
+	///
+	/// The defaults are implementation-defined and should not be relied upon.
+	fn default() -> Self {
+		Self{
+			max_token_length: 8192,
+		}
+	}
+}
+
 fn resolve_named_entity(name: &str, into: &mut String) -> Result<()> {
 	// amp, lt, gt, apos, quot
 	match name {
@@ -470,14 +481,14 @@ pub struct Lexer {
 	err: Option<Box<Error>>,
 }
 impl Lexer {
-	/// Construct a new Lexer based on [`LexerOptions::defaults()`].
-	pub fn new() -> Lexer {
-		Lexer::with_options(LexerOptions::defaults())
+	/// Construct a new Lexer based on [`LexerOptions::default()`].
+	pub fn new() -> Self {
+		Self::with_options(LexerOptions::default())
 	}
 
 	/// Construct a new Lexer with the given options.
-	pub fn with_options(opts: LexerOptions) -> Lexer {
-		Lexer {
+	pub fn with_options(opts: LexerOptions) -> Self {
+		Self {
 			state: State::Content(ContentState::Initial),
 			scratchpad: String::new(),
 			swap: String::new(),
@@ -598,7 +609,7 @@ impl Lexer {
 		self.flush_scratchpad(|s| {
 			#[cfg(debug_assertions)]
 			{
-				Name::from_str(s).unwrap()
+				s.try_into().unwrap()
 			}
 			#[cfg(not(debug_assertions))]
 			unsafe {
@@ -611,7 +622,7 @@ impl Lexer {
 		self.flush_scratchpad(|s| {
 			#[cfg(debug_assertions)]
 			{
-				CData::from_str(s).unwrap()
+				s.try_into().unwrap()
 			}
 			#[cfg(not(debug_assertions))]
 			unsafe {
@@ -1395,6 +1406,7 @@ pub trait Sink {
 #[cfg(test)]
 mod tests {
 	use super::*;
+	use std::io;
 	use std::fmt;
 	use std::error;
 	use crate::lexer::read::DecodingReader;
@@ -1537,7 +1549,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).err().unwrap();
 
-		assert_eq!(sink.dest[1], Token::Name(TokenMetrics{start: 6, end: 13}, Name::from_str("version").unwrap()));
+		assert_eq!(sink.dest[1], Token::Name(TokenMetrics{start: 6, end: 13}, "version".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1557,7 +1569,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).err().unwrap();
 
-		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, CData::from_str("1.0").unwrap()));
+		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, "1.0".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1567,7 +1579,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).err().unwrap();
 
-		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, CData::from_str("1.0").unwrap()));
+		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, "1.0".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1589,12 +1601,12 @@ mod tests {
 
 		assert!(result.is_ok());
 		assert_eq!(sink.dest[0], Token::XMLDeclStart(TokenMetrics{start: 0, end: 5}));
-		assert_eq!(sink.dest[1], Token::Name(TokenMetrics{start: 6, end: 13}, Name::from_str("version").unwrap()));
+		assert_eq!(sink.dest[1], Token::Name(TokenMetrics{start: 6, end: 13}, "version".try_into().unwrap()));
 		assert_eq!(sink.dest[2], Token::Eq(TokenMetrics{start: 13, end: 14}));
-		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, CData::from_str("1.0").unwrap()));
-		assert_eq!(sink.dest[4], Token::Name(TokenMetrics{start: 20, end: 28}, Name::from_str("encoding").unwrap()));
+		assert_eq!(sink.dest[3], Token::AttributeValue(TokenMetrics{start: 14, end: 19}, "1.0".try_into().unwrap()));
+		assert_eq!(sink.dest[4], Token::Name(TokenMetrics{start: 20, end: 28}, "encoding".try_into().unwrap()));
 		assert_eq!(sink.dest[5], Token::Eq(TokenMetrics{start: 28, end: 29}));
-		assert_eq!(sink.dest[6], Token::AttributeValue(TokenMetrics{start: 29, end: 36}, CData::from_str("utf-8").unwrap()));
+		assert_eq!(sink.dest[6], Token::AttributeValue(TokenMetrics{start: 29, end: 36}, "utf-8".try_into().unwrap()));
 		assert_eq!(sink.dest[7], Token::XMLDeclEnd(TokenMetrics{start: 36, end: 38}));
 	}
 
@@ -1605,7 +1617,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).err().unwrap();
 
-		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, Name::from_str("element").unwrap()));
+		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, "element".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1615,7 +1627,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).unwrap();
 
-		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, Name::from_str("element").unwrap()));
+		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, "element".try_into().unwrap()));
 		assert_eq!(sink.dest[1], Token::ElementHeadClose(TokenMetrics{start: 8, end: 10}));
 	}
 
@@ -1626,7 +1638,7 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).unwrap();
 
-		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, Name::from_str("element").unwrap()));
+		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, "element".try_into().unwrap()));
 		assert_eq!(sink.dest[1], Token::ElementHFEnd(TokenMetrics{start: 8, end: 9}));
 	}
 
@@ -1637,9 +1649,9 @@ mod tests {
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).unwrap();
 
-		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, Name::from_str("element").unwrap()));
+		assert_eq!(sink.dest[0], Token::ElementHeadStart(TokenMetrics{start: 0, end: 8}, "element".try_into().unwrap()));
 		assert_eq!(sink.dest[1], Token::ElementHFEnd(TokenMetrics{start: 8, end: 9}));
-		assert_eq!(sink.dest[2], Token::ElementFootStart(TokenMetrics{start: 9, end: 18}, Name::from_str("element").unwrap()));
+		assert_eq!(sink.dest[2], Token::ElementFootStart(TokenMetrics{start: 9, end: 18}, "element".try_into().unwrap()));
 		assert_eq!(sink.dest[3], Token::ElementHFEnd(TokenMetrics{start: 18, end: 19}));
 	}
 
@@ -1652,18 +1664,18 @@ mod tests {
 
 		let mut iter = sink.dest.iter();
 		assert!(matches!(iter.next().unwrap(), Token::ElementHeadStart(_, nm) if nm == "element"));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 9, end: 10}, Name::from_str("x").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 9, end: 10}, "x".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::Eq(_)));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 11, end: 16}, CData::from_str("foo").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 17, end: 18}, Name::from_str("y").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 11, end: 16}, "foo".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 17, end: 18}, "y".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::Eq(_)));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 19, end: 24}, CData::from_str("bar").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 25, end: 30}, Name::from_str("xmlns").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 19, end: 24}, "bar".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 25, end: 30}, "xmlns".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::Eq(_)));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 31, end: 36}, CData::from_str("baz").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 37, end: 46}, Name::from_str("xmlns:abc").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 31, end: 36}, "baz".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 37, end: 46}, "xmlns:abc".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::Eq(_)));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 47, end: 54}, CData::from_str("fnord").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 47, end: 54}, "fnord".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 54, end: 55}));
 	}
 
@@ -1677,7 +1689,7 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		assert!(matches!(iter.next().unwrap(), Token::ElementHeadStart(_, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(_)));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 18}, CData::from_str("Hello World!").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 18}, "Hello World!".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::ElementFootStart(_, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(_)));
 	}
@@ -1692,7 +1704,7 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		assert!(matches!(iter.next().unwrap(), Token::ElementHeadStart(_, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6})));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 11}, CData::from_str("&").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 11}, "&".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 11, end: 17}, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(_)));
 	}
@@ -1707,7 +1719,7 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		assert!(matches!(iter.next().unwrap(), Token::ElementHeadStart(_, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6})));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 11}, CData::from_str("<").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 11}, "<".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 11, end: 17}, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(_)));
 	}
@@ -1722,7 +1734,7 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		assert!(matches!(iter.next().unwrap(), Token::ElementHeadStart(_, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6})));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 12}, CData::from_str(">").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 12}, ">".try_into().unwrap()));
 		assert!(matches!(iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 12, end: 18}, nm) if nm == "root"));
 		assert!(matches!(iter.next().unwrap(), Token::ElementHFEnd(_)));
 	}
@@ -1794,7 +1806,7 @@ mod tests {
 		iter.next().unwrap();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 9, end: 10}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 10, end: 17}, CData::from_str("&").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 10, end: 17}, "&".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1808,7 +1820,7 @@ mod tests {
 		iter.next().unwrap();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 9, end: 10}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 10, end: 71}, CData::from_str("<example foo=\"bar\" baz='fnord'/>").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 10, end: 71}, "<example foo=\"bar\" baz='fnord'/>".try_into().unwrap()));
 	}
 
 	#[test]
@@ -1821,8 +1833,8 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6}));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 50}, CData::from_str("<example foo=\"bar\" baz='fnord'/>").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 50, end: 56}, Name::from_str("root").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 6, end: 50}, "<example foo=\"bar\" baz='fnord'/>".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 50, end: 56}, "root".try_into().unwrap()));
 		iter.next().unwrap();
 	}
 
@@ -1836,7 +1848,7 @@ mod tests {
 		let mut iter = sink.dest.iter();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6}));
-		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 18, end: 24}, Name::from_str("root").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 18, end: 24}, "root".try_into().unwrap()));
 		iter.next().unwrap();
 	}
 
@@ -1863,7 +1875,7 @@ mod tests {
 	fn lexer_lex_restrict_element_name_by_token_length() {
 		let src = &b"<foobar2342/>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		let result = stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink);
 
@@ -1874,7 +1886,7 @@ mod tests {
 	fn lexer_lex_restrict_attribute_name_by_token_length() {
 		let src = &b"<a foobar2342='foo'/>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		let result = stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink);
 
@@ -1885,7 +1897,7 @@ mod tests {
 	fn lexer_lex_restrict_attribute_value_by_token_length() {
 		let src = &b"<a b='foobar2342'/>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		let result = stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink);
 
@@ -1896,7 +1908,7 @@ mod tests {
 	fn lexer_lex_restrict_attribute_value_by_token_length_even_with_entities() {
 		let src = &b"<a b='foob&amp;r'/>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		let result = stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink);
 
@@ -1907,7 +1919,7 @@ mod tests {
 	fn lexer_lex_attribute_value_entities_do_only_count_for_expansion() {
 		let src = &b"<a b='foob&amp;'/>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink).unwrap();
 	}
@@ -1916,17 +1928,17 @@ mod tests {
 	fn lexer_lex_token_length_causes_text_nodes_to_be_split() {
 		let src = &b"<a>foo001foo002foo003</a>"[..];
 		let mut buffered = io::BufReader::with_capacity(1, src);
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut buffered, &mut sink).unwrap();
 
 		let mut iter = sink.dest.iter();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 2, end: 3}));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 3, end: 9}, CData::from_str("foo001").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 9, end: 15}, CData::from_str("foo002").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 15, end: 21}, CData::from_str("foo003").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 21, end: 24}, Name::from_str("a").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 3, end: 9}, "foo001".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 9, end: 15}, "foo002".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 15, end: 21}, "foo003".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 21, end: 24}, "a".try_into().unwrap()));
 		iter.next().unwrap();
 	}
 
@@ -2037,14 +2049,14 @@ mod tests {
 	#[test]
 	fn lexer_handles_closing_brackets_in_cdata_section() {
 		let mut src = &b"<a><![CDATA[]]]></a>"[..];
-		let mut lexer = Lexer::with_options(LexerOptions::defaults().max_token_length(6));
+		let mut lexer = Lexer::with_options(LexerOptions::default().max_token_length(6));
 		let mut sink = VecSink::new(128);
 		stream_to_sink_from_bytes(&mut lexer, &mut src, &mut sink).unwrap();
 
 		let mut iter = sink.dest.iter();
 		iter.next().unwrap();
-		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 2, end: 3}));	assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 3, end: 16}, CData::from_str("]").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 16, end: 19}, Name::from_str("a").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 2, end: 3}));	assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 3, end: 16}, "]".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 16, end: 19}, "a".try_into().unwrap()));
 		iter.next().unwrap();
 	}
 
@@ -2068,9 +2080,9 @@ mod tests {
 
 		let mut iter = sink.iter();
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclStart(TokenMetrics{start: 0, end: 5}));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 6, end: 13}, Name::from_str("version").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 6, end: 13}, "version".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 13, end: 14}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 14, end: 19}, CData::from_str("1.0").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 14, end: 19}, "1.0".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclEnd(TokenMetrics{start: 19, end: 21}));
 	}
 
@@ -2094,9 +2106,9 @@ mod tests {
 
 		let mut iter = sink.iter();
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclStart(TokenMetrics{start: 0, end: 5}));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 8, end: 15}, Name::from_str("version").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 8, end: 15}, "version".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 17, end: 18}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 20, end: 25}, CData::from_str("1.0").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 20, end: 25}, "1.0".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclEnd(TokenMetrics{start: 27, end: 29}));
 	}
 
@@ -2213,7 +2225,7 @@ mod tests {
 		let mut iter = toks.iter();
 		iter.next().unwrap();
 		assert_eq!(*iter.next().unwrap(), Token::ElementHFEnd(TokenMetrics{start: 5, end: 6}));
-		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 18, end: 24}, Name::from_str("root").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::ElementFootStart(TokenMetrics{start: 18, end: 24}, "root".try_into().unwrap()));
 		iter.next().unwrap();
 
 		let (toks, r) = lex(&b"<root><![CDATA[]]>&amp;</root>"[..], 128);
@@ -2222,7 +2234,7 @@ mod tests {
 		let mut iter = toks.iter();
 		iter.next().unwrap();
 		iter.next().unwrap();
-		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 18, end: 23}, CData::from_str("&").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Text(TokenMetrics{start: 18, end: 23}, "&".try_into().unwrap()));
 
 		let (toks, r) = lex(&b"<root><![CDATA[]]><![CDATA[]]]]>&gt;</root>"[..], 128);
 		r.unwrap();
@@ -2273,12 +2285,12 @@ mod tests {
 
 		let mut iter = sink.dest.iter();
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclStart(TokenMetrics{start: 0, end: 5}));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 6, end: 13}, Name::from_str("version").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 6, end: 13}, "version".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 13, end: 14}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 14, end: 19}, CData::from_str("1.0").unwrap()));
-		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 20, end: 28}, Name::from_str("encoding").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 14, end: 19}, "1.0".try_into().unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::Name(TokenMetrics{start: 20, end: 28}, "encoding".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::Eq(TokenMetrics{start: 28, end: 29}));
-		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 29, end: 36}, CData::from_str("utf-8").unwrap()));
+		assert_eq!(*iter.next().unwrap(), Token::AttributeValue(TokenMetrics{start: 29, end: 36}, "utf-8".try_into().unwrap()));
 		assert_eq!(*iter.next().unwrap(), Token::XMLDeclEnd(TokenMetrics{start: 36, end: 38}));
 		match iter.next().unwrap() {
 			Token::ElementHeadStart(tm, ..) => {
