@@ -22,66 +22,50 @@ times.
 
   Note that [`CData`] strings do not contain references or CDATA sections;
   those are expanded by the lexer.
+
+## Construction
+
+To construct [`str`]-like references at compile time, you can use the macros
+from the [`rxml_proc`](https://docs.rs/rxml_proc) crate. It offers
+`xml_cdata!`, `xml_name!` and `xml_ncname!` macros which convert a string
+literal into `&CDataStr`, `&NameStr` and `&NCNameStr` (respectively) with
+validation at compile time.
+
+In general, owned values are constructed using the [`std::convert::TryInto`]
+mechanism, from other string types. Supported source types are:
+
+* [`String`] (copies for [`Name`] and [`NCName`], moves for [`CData`])
+* [`smartstring::alias::String`] (copies for [`CData`], moves for [`Name`] and [`NCName])
+* [`str`] (copies for all types except the slice types)
+
+In addition, the following conversions can be done without extra checking and
+are possible through `.into()`:
+
+* [`NCName`] to [`Name`]
+* [`NCName`] to [`CData`]
+* [`Name`] to [`CData`]
+
+(and likewise for the corresponding Str types)
+
+The inverse directions are only available through `try_into`.
 */
 
 use std::ops::Deref;
 use std::fmt;
-use std::convert::TryFrom;
+use std::convert::{TryFrom, TryInto};
 use std::borrow::{Borrow, ToOwned, Cow};
-use crate::selectors;
-use crate::selectors::{CharSelector, CodepointRanges};
 use crate::error::{NWFError, WFError, ERRCTX_UNKNOWN};
 use smartstring::alias::String as SmartString;
-
-static CDATA_ANTI_SELECTOR: CodepointRanges = CodepointRanges(selectors::INVALID_XML_CDATA_RANGES);
-
-fn validate_name(s: &str) -> Result<(), WFError> {
-	let mut chars = s.chars();
-	match chars.next() {
-		// must have at least one char
-		None => return Err(WFError::InvalidSyntax("Name must have at least one Char")),
-		Some(c) => if !selectors::CLASS_XML_NAMESTART.select(c) {
-			return Err(WFError::UnexpectedChar(ERRCTX_UNKNOWN, c, None))
-		}
-	}
-	for ch in chars {
-		if !selectors::CLASS_XML_NAME.select(ch) {
-			return Err(WFError::UnexpectedChar(ERRCTX_UNKNOWN, ch, None))
-		}
-	}
-	Ok(())
-}
-
-fn validate_ncname(s: &str) -> Result<(), WFError> {
-	let mut chars = s.chars();
-	match chars.next() {
-		// must have at least one char
-		None => return Err(WFError::InvalidSyntax("Name must have at least one Char")),
-		Some(c) => if !selectors::CLASS_XML_NAMESTART.select(c) || c == ':' {
-			return Err(WFError::UnexpectedChar(ERRCTX_UNKNOWN, c, None))
-		}
-	}
-	for ch in chars {
-		if !selectors::CLASS_XML_NAME.select(ch) || ch == ':' {
-			return Err(WFError::UnexpectedChar(ERRCTX_UNKNOWN, ch, None))
-		}
-	}
-	Ok(())
-}
-
-fn validate_cdata(s: &str) -> Result<(), WFError> {
-	for ch in s.chars() {
-		if CDATA_ANTI_SELECTOR.select(ch) {
-			return Err(WFError::UnexpectedChar(ERRCTX_UNKNOWN, ch, None))
-		}
-	}
-	Ok(())
-}
+use rxml_validation::selectors;
+use rxml_validation::selectors::CharSelector;
+pub use rxml_validation::{validate_name, validate_ncname, validate_cdata};
 
 /// String which conforms to the Name production of XML 1.0.
 ///
 /// [`Name`] corresponds to a (restricted) [`String`]. For a [`str`]-like type
-/// with the same restrictions, see [`NameStr`].
+/// with the same restrictions, see [`NameStr`]. `&NameStr` can be created
+/// from a string literal at compile time using the `xml_name` macro from
+/// [`rxml_proc`](https://docs.rs/rxml_proc).
 ///
 /// Since [`Name`] derefs to [`String`], all (non-mutable) methods from
 /// [`String`] are available.
@@ -112,10 +96,9 @@ impl Name {
 	/// This function enforces that the given string conforms to the `Name`
 	/// production of XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_string<T: Into<String>>(s: T) -> Result<Name, WFError> {
-		let s = s.into();
-		validate_name(&s)?;
-		Ok(Name(s.into()))
+		s.into().try_into()
 	}
 
 	/// Wrap a given [`smartstring::SmartString`] in a [`Name`].
@@ -123,10 +106,9 @@ impl Name {
 	/// This function enforces that the given string conforms to the `Name`
 	/// production of XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_smartstring<T: Into<SmartString>>(s: T) -> Result<Name, WFError> {
-		let s = s.into();
-		validate_name(&s)?;
-		Ok(Name(s))
+		s.into().try_into()
 	}
 
 	/// Copy a given [`str`]-like into a new [`Name`].
@@ -134,10 +116,9 @@ impl Name {
 	/// This function enforces that the given string conforms to the `Name`
 	/// production of XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_str<T: AsRef<str>>(s: T) -> Result<Name, WFError> {
-		let s = s.as_ref();
-		validate_name(s)?;
-		Ok(Name(s.into()))
+		s.as_ref().try_into()
 	}
 
 	/// Split the name at a colon, if it exists.
@@ -190,19 +171,31 @@ impl Name {
 	}
 
 	/// Construct a Name without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid Name.
 	pub unsafe fn from_str_unchecked<T: AsRef<str>>(s: T) -> Name {
 		Name(s.as_ref().into())
 	}
 
 	/// Construct a Name without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid Name.
 	pub unsafe fn from_string_unchecked<T: Into<String>>(s: T) -> Name {
 		Name(s.into().into())
 	}
 
 	/// Construct a Name without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid Name.
 	pub unsafe fn from_smartstring_unchecked<T: Into<SmartString>>(s: T) -> Name {
 		Name(s.into())
 	}
@@ -336,7 +329,8 @@ impl TryFrom<SmartString> for Name  {
 	type Error = WFError;
 
 	fn try_from(other: SmartString) -> Result<Self, Self::Error> {
-		Name::from_smartstring(other)
+		validate_name(&other)?;
+		Ok(Name(other))
 	}
 }
 
@@ -344,7 +338,8 @@ impl TryFrom<String> for Name  {
 	type Error = WFError;
 
 	fn try_from(other: String) -> Result<Self, Self::Error> {
-		Name::from_string(other)
+		validate_name(&other)?;
+		Ok(Name(other.into()))
 	}
 }
 
@@ -352,7 +347,8 @@ impl TryFrom<&str> for Name  {
 	type Error = WFError;
 
 	fn try_from(other: &str) -> Result<Self, Self::Error> {
-		Name::from_str(other)
+		validate_name(other)?;
+		Ok(Name(other.into()))
 	}
 }
 
@@ -368,6 +364,9 @@ impl fmt::Display for Name {
 /// type with the same restrictions as well as the formal definition of those
 /// restrictions, see [`Name`].
 ///
+/// `&NameStr` can be created from a string literal at compile time using the
+/// `xml_name` macro from [`rxml_proc`](https://docs.rs/rxml_proc).
+///
 /// Since [`NameStr`] derefs to [`str`], all (non-mutable) methods from
 /// [`str`] are available.
 #[derive(Hash, PartialEq)]
@@ -380,9 +379,9 @@ impl NameStr {
 	/// This function enforces that the given string conforms to the `Name`
 	/// production of XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation or xml_name! macro instead")]
 	pub fn from_str<'x>(s: &'x str) -> Result<&'x NameStr, WFError> {
-		validate_name(s)?;
-		Ok(unsafe { std::mem::transmute(s) })
+		s.try_into()
 	}
 
 	/// Copy the NameStr into a new Name.
@@ -391,7 +390,11 @@ impl NameStr {
 	}
 
 	/// Construct a NameStr without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid Name.
 	pub unsafe fn from_str_unchecked<'x>(s: &'x str) -> &'x NameStr {
 		std::mem::transmute(s)
 	}
@@ -475,6 +478,15 @@ impl From<&NameStr> for Name {
 	}
 }
 
+impl<'x> TryFrom<&'x str> for &'x NameStr {
+	type Error = WFError;
+
+	fn try_from(other: &'x str) -> Result<Self, Self::Error> {
+		validate_name(other)?;
+		Ok(unsafe { std::mem::transmute(other) })
+	}
+}
+
 impl fmt::Display for NameStr {
 	fn fmt<'f>(&self, f: &'f mut fmt::Formatter) -> fmt::Result {
 		f.write_str(&self.0)
@@ -484,7 +496,9 @@ impl fmt::Display for NameStr {
 /// String which conforms to the NCName production of Namespaces in XML 1.0.
 ///
 /// [`NCName`] corresponds to a (restricted) [`String`]. For a [`str`]-like
-/// type with the same restrictions, see [`NCNameStr`].
+/// type with the same restrictions, see [`NCNameStr`]. `&NCNameStr` can be
+/// created from a string literal at compile time using the `xml_ncname` macro
+/// from [`rxml_proc`](https://docs.rs/rxml_proc).
 ///
 /// Since [`NCName`] derefs to [`String`], all (non-mutable) methods from
 /// [`String`] are available.
@@ -569,19 +583,31 @@ impl NCName {
 	}
 
 	/// Construct an NCName without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid NCName.
 	pub unsafe fn from_str_unchecked<T: AsRef<str>>(s: T) -> NCName {
 		NCName(s.as_ref().into())
 	}
 
 	/// Construct an NCName without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid NCName.
 	pub unsafe fn from_string_unchecked<T: Into<String>>(s: T) -> NCName {
 		NCName(s.into().into())
 	}
 
 	/// Construct an NCName without enforcing anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid NCName.
 	pub unsafe fn from_smartstring_unchecked<T: Into<SmartString>>(s: T) -> NCName {
 		NCName(s.into())
 	}
@@ -741,6 +767,9 @@ impl fmt::Display for NCName {
 /// type with the same restrictions as well as the formal definition of those
 /// restrictions, see [`NCName`].
 ///
+/// `&NCNameStr` can be created from a string literal at compile time using
+/// the `xml_ncname` macro from [`rxml_proc`](https://docs.rs/rxml_proc).
+///
 /// Since [`NCNameStr`] derefs to [`str`], all (non-mutable) methods from
 /// [`str`] are available.
 #[derive(Hash, PartialEq)]
@@ -753,6 +782,7 @@ impl NCNameStr {
 	/// This function enforces that the given string conforms to the `NCName`
 	/// production of Namespaces in XML 1.0. If those conditions are not met,
 	/// an error is returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation or xml_ncname! macro instead")]
 	pub fn from_str<'x>(s: &'x str) -> Result<&'x NCNameStr, WFError> {
 		validate_name(s)?;
 		Ok(unsafe { std::mem::transmute(s) })
@@ -764,7 +794,11 @@ impl NCNameStr {
 	}
 
 	/// Construct a NCNameStr without checking anything
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// a valid NCName.
 	pub unsafe fn from_str_unchecked<'x>(s: &'x str) -> &'x NCNameStr {
 		std::mem::transmute(s)
 	}
@@ -857,7 +891,9 @@ impl fmt::Display for NCNameStr {
 /// String which consists only of XML 1.0 Chars.
 ///
 /// [`CData`] corresponds to a (restricted) [`String`]. For a [`str`]-like
-/// type with the same restrictions, see [`CDataStr`].
+/// type with the same restrictions, see [`CDataStr`]. `&CDataStr` can be
+/// created from a string literal at compile time using the `xml_cdata` macro
+/// from [`rxml_proc`](https://docs.rs/rxml_proc).
 ///
 /// Since [`CData`] derefs to [`String`], all (non-mutable) methods from
 /// [`String`] are available.
@@ -878,6 +914,13 @@ impl fmt::Display for NCNameStr {
 /// This is a Unicode scalar value, minus ASCII control characters except
 /// Tab (`\x09`), CR (`\x0d`) and LF (`\x0a`), the BOM (`\u{fffe}`) and
 /// whatever `\u{ffff}` is.
+///
+/// # Escaping
+///
+/// [`CData`] objects do not contain references or CDATA sections as those are
+/// expanded by the lexer. This implies that `CData` objects are not safe to
+/// just verbatimly copy into an XML document; additional escaping may be
+/// necessary.
 #[derive(Hash, PartialEq, Debug, Clone)]
 pub struct CData(String);
 
@@ -887,10 +930,9 @@ impl CData {
 	/// This function enforces that the chars in the string conform to `Char`
 	/// as defined in XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_string<T: Into<String>>(s: T) -> Result<CData, WFError> {
-		let s = s.into();
-		validate_cdata(&s)?;
-		Ok(CData(s))
+		s.into().try_into()
 	}
 
 	/// Wrap a given [`smartstring::SmartString`] in a [`CData`].
@@ -898,10 +940,9 @@ impl CData {
 	/// This function enforces that the chars in the string conform to `Char`
 	/// as defined in XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_smartstring<T: Into<SmartString>>(s: T) -> Result<CData, WFError> {
-		let s = s.into();
-		validate_cdata(&s)?;
-		Ok(CData(s.into()))
+		s.into().try_into()
 	}
 
 	/// Copy a given [`str`]-like into a new [`NCName`].
@@ -909,10 +950,9 @@ impl CData {
 	/// This function enforces that the chars in the string conform to `Char`
 	/// as defined in XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 	pub fn from_str<T: AsRef<str>>(s: T) -> Result<CData, WFError> {
-		let s = s.as_ref();
-		validate_cdata(s)?;
-		Ok(CData(s.to_string()))
+		s.as_ref().try_into()
 	}
 
 	pub fn as_cdata_str(&self) -> &CDataStr {
@@ -925,19 +965,31 @@ impl CData {
 	}
 
 	/// Construct a CData without checking anything.
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// valid CData.
 	pub unsafe fn from_str_unchecked<T: AsRef<str>>(s: T) -> CData {
 		CData(s.as_ref().into())
 	}
 
 	/// Construct a CData without checking anything.
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// valid CData.
 	pub unsafe fn from_smartstring_unchecked<T: Into<SmartString>>(s: T) -> CData {
 		CData(s.into().into())
 	}
 
 	/// Construct a CData without checking anything.
-	#[doc(hidden)]
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// valid CData.
 	pub unsafe fn from_string_unchecked<T: Into<String>>(s: T) -> CData {
 		CData(s.into())
 	}
@@ -1071,7 +1123,8 @@ impl TryFrom<SmartString> for CData {
 	type Error = WFError;
 
 	fn try_from(other: SmartString) -> Result<Self, Self::Error> {
-		CData::from_smartstring(other)
+		validate_cdata(&other)?;
+		Ok(CData(other.into()))
 	}
 }
 
@@ -1079,7 +1132,8 @@ impl TryFrom<String> for CData {
 	type Error = WFError;
 
 	fn try_from(other: String) -> Result<Self, Self::Error> {
-		CData::from_string(other)
+		validate_cdata(&other)?;
+		Ok(CData(other))
 	}
 }
 
@@ -1087,7 +1141,8 @@ impl TryFrom<&str> for CData  {
 	type Error = WFError;
 
 	fn try_from(other: &str) -> Result<Self, Self::Error> {
-		CData::from_str(other)
+		validate_cdata(other)?;
+		Ok(CData(other.to_string()))
 	}
 }
 
@@ -1103,6 +1158,9 @@ impl fmt::Display for CData {
 /// type with the same restrictions as well as the formal definition of those
 /// restrictions, see [`CData`].
 ///
+/// `&CDataStr` can be created from a string literal at compile time using the
+/// `xml_cdata` macro from [`rxml_proc`](https://docs.rs/rxml_proc).
+///
 /// Since [`CDataStr`] derefs to [`str`], all (non-mutable) methods from
 /// [`str`] are available.
 #[derive(Hash, PartialEq)]
@@ -1115,9 +1173,9 @@ impl CDataStr {
 	/// This function enforces that the chars in the string conform to `Char`
 	/// as defined in XML 1.0. If those conditions are not met, an error is
 	/// returned.
+	#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation  or xml_cdata! macro instead")]
 	pub fn from_str<'x>(s: &'x str) -> Result<&'x CDataStr, WFError> {
-		validate_cdata(s)?;
-		Ok(unsafe { std::mem::transmute(s) })
+		s.try_into()
 	}
 
 	/// Copy the CDataStr into a new CData
@@ -1125,7 +1183,12 @@ impl CDataStr {
 		unsafe { CData::from_string_unchecked(self.0.to_string()) }
 	}
 
-	#[doc(hidden)]
+	/// Construct a CDataStr without checking anything
+	///
+	/// # Safety
+	///
+	/// The caller is responsible for ensuring that the passed data is in fact
+	/// valid CData.
 	pub unsafe fn from_str_unchecked<'x>(s: &'x str) -> &'x CDataStr {
 		std::mem::transmute(s)
 	}
@@ -1209,19 +1272,28 @@ impl From<&CDataStr> for CData {
 	}
 }
 
+impl<'x> TryFrom<&'x str> for &'x CDataStr {
+	type Error = WFError;
+
+	fn try_from(other: &'x str) -> Result<Self, Self::Error> {
+		validate_cdata(other)?;
+		Ok(unsafe { std::mem::transmute(other) })
+	}
+}
+
 #[cfg(test)]
 mod tests {
 	use super::*;
 
 	#[test]
 	fn split_name_rejects_localname_with_non_namestart_first_char() {
-		let nm = Name::from_str("foo:-bar").unwrap();
+		let nm: Name = "foo:-bar".try_into().unwrap();
 		let result = nm.split_name();
 		assert!(matches!(result.err().unwrap(), NWFError::InvalidLocalName(_)));
 	}
 
 	#[test]
 	fn cdatastr_allows_slashes() {
-		CDataStr::from_str("http://www.w3.org/XML/1998/namespace").unwrap();
+		let _: &CDataStr = "http://www.w3.org/XML/1998/namespace".try_into().unwrap();
 	}
 }
